# Author    : Nathan Chen
# Date      : 11-Mar-2024


import os
import streamlit.components.v1 as components
import streamlit as st
from typing import Literal, Optional, Union, Any, Dict, overload


_RELEASE = True


if not _RELEASE:
    _componentFunc = components.declare_component("rsa_auth_ui", url="http://localhost:3001")
else:
    parent_dir = os.path.dirname(os.path.abspath(__file__))
    build_dir = os.path.join(parent_dir, "frontend/build")
    _componentFunc = components.declare_component("rsa_auth_ui", path=build_dir)

Object = Dict[str, Any]
ConfigType = Dict[str, Object]
FormLocation = Literal['main', 'sidebar']


def createComponent(id: str, publicKey: Optional[str], default: Optional[Object], configs: Optional[Object], defaultLocation: FormLocation) -> Union[Object, str, None]:
    location = defaultLocation if configs is None or 'location' not in configs else configs['location']
    if location is 'sidebar':
        with st.sidebar:
            result = _componentFunc(id = id, publicKey=publicKey, default=default, configs=configs)
    else:
        result = _componentFunc(id = id, publicKey=publicKey, default=default, configs=configs)

    if publicKey is not None and type(result) is not str: return None
    return result


@overload
def signinForm(publicKey: None, default: Optional[Object] = None, configs: Optional[Object] = None) -> Optional[Object]: pass
@overload
def signinForm(publicKey: str, default: Optional[Object] = None, configs: Optional[Object] = None) -> Optional[str]: pass
def signinForm(publicKey: Optional[str] = None, default: Optional[Object] = None, configs: Optional[Object] = None) -> Union[Object, str, None]:
    return createComponent('signin', publicKey, default, configs, 'main')


@overload
def signoutForm(publicKey: None, default: Optional[Object] = None, configs: Optional[Object] = None) -> Optional[Object]: pass
@overload
def signoutForm(publicKey: str, default: Optional[Object] = None, configs: Optional[Object] = None) -> Optional[str]: pass
def signoutForm(publicKey: Optional[str] = None, default: Optional[Object] = None, configs: Optional[Object] = None) -> Union[Object, str, None]:
    return createComponent('signout', publicKey, default, configs, 'sidebar')


# Author    : Nathan Chen
# Date      : 11-Mar-2024


from .encryption import Encryptor
from .events import SigninEvent, CancelSigninEvent, SignoutEvent, CancelSignoutEvent, EventInfo, getEvent
from .form import FormLocation, ConfigType, Object, signinForm, signoutForm


# Author    : Nathan Chen
# Date      : 11-Mar-2024


import rsa
import os
import base64
import json
from typing import Dict, Any


class Encryptor:
    privateKey: rsa.PrivateKey
    publicKeyPem: str

    @classmethod
    def generateNew(cls, nbits: int):
        (publicKey, privateKey) = rsa.newkeys(nbits)
        encryptor = cls()
        encryptor.privateKey = privateKey
        encryptor.publicKeyPem = publicKey.save_pkcs1().decode()
        return encryptor

    @classmethod
    def getKeyPath(cls, path: str, name: str):
        publicKeyPath = os.path.join(path, f'{name}.pub')
        privateKeyPath = os.path.join(path, name)
        return (publicKeyPath, privateKeyPath)        
    
    @classmethod
    def load(cls, path: str, name: str):
        (publicKeyPath, privateKeyPath) = cls.getKeyPath(path, name)
        encryptor = cls()
        
        with open(publicKeyPath, 'r') as f:
            encryptor.publicKeyPem = f.read()
        
        with open(privateKeyPath, 'rb') as f:
            bytes = f.read()
            encryptor.privateKey = rsa.PrivateKey.load_pkcs1(bytes)

        return encryptor
    
    def save(self, path: str, name: str):
        (publicKeyPath, privateKeyPath) = self.getKeyPath(path, name)

        with open(publicKeyPath, 'w') as f:
            f.write(self.publicKeyPem)

        with open(privateKeyPath, 'wb') as f:
            bytes = self.privateKey.save_pkcs1()
            f.write(bytes)

    def decrypt(self, base64Str: str) -> Dict[str, Any]:
        encrypted = base64.b64decode(base64Str)
        decrypted = rsa.decrypt(encrypted, self.privateKey)
        _json = decrypted.decode()
        _dict = json.loads(_json)
        return _dict
    


# Author    : Nathan Chen
# Date      : 11-Mar-2024


from typing import Literal


EventInfo = Literal['signin', 'cancelSignin', 'signout', 'cancelSignout']


class Event:
    event: EventInfo

    def __init__(self, event: EventInfo) -> None:
        self.event = event


class SigninEvent(Event):
    event: EventInfo = 'signin'
    username: str
    password: str
    remember: bool

    def __init__(self) -> None:
        super().__init__('signin')

class CancelSigninEvent(Event):
    def __init__(self) -> None:
        super().__init__('cancelSignin')

class SignoutEvent(Event):
    def __init__(self) -> None:
        super().__init__('signout')

class CancelSignoutEvent(Event):
    def __init__(self) -> None:
        super().__init__('cancelSignout')


def getEvent(value):
    if type(value) is not dict: return None
    info: EventInfo = value['event']
    event = None
    if info == 'signin':
        event = SigninEvent()
        event.__dict__ = value
    elif info == 'cancelSignin':
        event = CancelSignoutEvent()
    elif info == 'signout':
        event = SignoutEvent()
    elif info == 'cancelSignout':
        event = CancelSignoutEvent()

    return event


# Author   : Nathan Chen
# Date     : 08-Mar-2024


from pathlib import Path
from setuptools import setup, find_packages

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding='UTF-8')

setup(
    name='streamlit-rsa-auth-ui',
    version='0.0.1',
    author='Nathan Chen',
    author_email='nathan.chen.198@gmail.com',
    description='Streamlit RSA Authenticator UI',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/NathanChen198/streamlit-rsa-auth-ui',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.8",
    install_requires=[
        # By definition, a Custom Component depends on Streamlit.
        # If your component has other Python dependencies, list
        # them here.
        "streamlit >= 0.63",
        "rsa >= 4.5"
    ],
    extras_require={
        "devel": [
            "wheel",
            "pytest==7.4.0",
            "playwright==1.39.0",
            "requests==2.31.0",
            "pytest-playwright-snapshot==1.0",
            "pytest-rerunfailures==12.0",
        ]
    }
)